# -*- coding: utf-8 -*-

import pandas as pd
import numpy as np
import math


def accumulate_timing_risk(annualized_exante_period_timing_series):
    return annualized_exante_period_timing_series.mean()


def accumulate_management_risk(annualized_exante_period_management_df_dict):
    trading_day_list = sorted(annualized_exante_period_management_df_dict.keys())
    annualized_exante_period_management_df = pd.DataFrame(
        annualized_exante_period_management_df_dict[trading_day_list[0]])
    for trading_day in trading_day_list[1:]:
        annualized_exante_period_management_df += annualized_exante_period_management_df_dict[trading_day]
    annualized_exante_period_management_df /= len(trading_day_list)

    return annualized_exante_period_management_df


def compute_attr_linking_coef(period_return_df):
    R_p_t = period_return_df['portfolio']
    R_b_t = period_return_df['benchmark']
    R_p = np.prod(R_p_t.add(1.0)) - 1.0
    R_b = np.prod(R_b_t.add(1.0)) - 1.0
    k = (np.log(1.0 + R_p) - np.log(1.0 + R_b)) / (R_p - R_b)
    R_p_t_arr = R_p_t.values
    R_b_t_arr = R_b_t.values
    k_t = (np.log((1.0 + R_p_t_arr).tolist()) - np.log((1.0 + R_b_t_arr).tolist())) / (R_p_t_arr - R_b_t_arr)
    beta_arr = np.nan_to_num(k_t / k if k != 0.0 else [0.0] * len(k_t))
    beta_series = pd.Series(index=period_return_df.index, data=beta_arr)
    beta_series.index.name = 'date'

    return beta_series


def accumulate_style_factor_return(q, total_return_df, annualized_realized_period_style_factor_series_dict):
    trade_date_list = sorted(annualized_realized_period_style_factor_series_dict.keys())
    first_trade_date = trade_date_list[0]
    unannualized_realized_accumulated_style_factor_series_dict = dict()
    first_annualized_realized_period_style_factor_series = \
        annualized_realized_period_style_factor_series_dict[first_trade_date]
    unannualized_realized_accumulated_style_factor_series_dict[first_trade_date] = \
        first_annualized_realized_period_style_factor_series / q
    total_return_dict = total_return_df.to_dict()
    R_T_P = total_return_dict[first_trade_date]['portfolio'] / q
    R_T_B = total_return_dict[first_trade_date]['benchmark'] / q
    C_T = (R_T_P - R_T_B) / (np.log(1 + R_T_P) - np.log(1 + R_T_B))
    X_t = first_annualized_realized_period_style_factor_series
    for i in range(1, len(trade_date_list)):
        X_pre_t = X_t / q * i
        X_pre_t_divide_C_T = X_pre_t.divide(C_T).fillna(0.)
        trading_day = trade_date_list[i]
        annualized_realized_period_style_factor_series_t = \
            annualized_realized_period_style_factor_series_dict[trading_day]
        r_sector_t_series = annualized_realized_period_style_factor_series_t.divide(q)
        r_p_t = total_return_dict[trading_day]['portfolio'] / q
        r_b_t = total_return_dict[trading_day]['benchmark'] / q
        r_A_t = r_p_t - r_b_t
        if r_A_t == 0:
            unannualized_realized_accumulated_style_factor_series_dict[trading_day] = X_pre_t
            continue
        alpha_t = r_sector_t_series.divide(r_A_t)
        R_T_P = (1 + R_T_P) * (1 + r_p_t) - 1  # R_T+1_P
        R_T_B = (1 + R_T_B) * (1 + r_b_t) - 1  # R_T+1_B
        C_T = (R_T_P - R_T_B) / (np.log(1 + R_T_P) - np.log(1 + R_T_B))  # C_T+1
        X_t = (X_pre_t_divide_C_T * C_T + C_T * np.log((1 + r_p_t) / (1 + r_b_t)) * alpha_t) * q / (i + 1)
        unannualized_realized_accumulated_style_factor_series_dict[trading_day] = X_t / q * (i + 1)

    return unannualized_realized_accumulated_style_factor_series_dict


def accumulate_specific_return(q, total_return_df, annualized_realized_period_specific_return_series):
    trade_date_list = sorted(annualized_realized_period_specific_return_series.index)
    first_trade_date = trade_date_list[0]
    unannualized_realized_accumulated_specific_return_series = pd.Series(index=trade_date_list)
    unannualized_realized_accumulated_specific_return_series[first_trade_date] = \
        annualized_realized_period_specific_return_series[first_trade_date] / q
    total_return_dict = total_return_df.to_dict()
    R_T_P = total_return_dict[first_trade_date]['portfolio'] / q
    R_T_B = total_return_dict[first_trade_date]['benchmark'] / q
    C_T = (R_T_P - R_T_B) / (np.log(1 + R_T_P) - np.log(1 + R_T_B))
    X_t = annualized_realized_period_specific_return_series[first_trade_date]
    for i in range(1, len(trade_date_list)):
        X_pre_t = X_t / q * i
        X_pre_t_divide_C_T = X_pre_t / C_T
        if np.isnan(X_pre_t_divide_C_T):
            X_pre_t_divide_C_T = 0.
        trading_day = trade_date_list[i]
        annualized_realized_period_specific_return_t = annualized_realized_period_specific_return_series[trading_day]
        r_p_t = total_return_dict[trading_day]['portfolio'] / q
        r_b_t = total_return_dict[trading_day]['benchmark'] / q
        r_A_t = r_p_t - r_b_t
        if r_A_t == 0:
            unannualized_realized_accumulated_specific_return_series[trading_day] = X_pre_t
            continue
        alpha_t = annualized_realized_period_specific_return_t / q / r_A_t
        R_T_P = (1 + R_T_P) * (1 + r_p_t) - 1  # R_T+1_P
        R_T_B = (1 + R_T_B) * (1 + r_b_t) - 1  # R_T+1_B
        C_T = (R_T_P - R_T_B) / (np.log(1 + R_T_P) - np.log(1 + R_T_B))  # C_T+1
        X_t = (X_pre_t_divide_C_T * C_T + C_T * np.log((1 + r_p_t) / (1 + r_b_t)) * alpha_t) * q / (i + 1)
        unannualized_realized_accumulated_specific_return_series[trading_day] = X_t / q * (i + 1)

    return unannualized_realized_accumulated_specific_return_series


def accumulate_style_factor_risk(annualized_exante_period_style_factor_series_dict):
    trading_day_list = sorted(annualized_exante_period_style_factor_series_dict.keys())
    annualized_exante_accumulated_style_factor_series = \
        pd.Series(annualized_exante_period_style_factor_series_dict[trading_day_list[0]])
    for trading_day in trading_day_list[1:]:
        annualized_exante_accumulated_style_factor_series += \
            annualized_exante_period_style_factor_series_dict[trading_day]
    annualized_exante_accumulated_style_factor_series /= len(trading_day_list)
    annualized_exante_accumulated_style_factor_series.name = 'style_factor_annualized_exante_risk'
    return annualized_exante_accumulated_style_factor_series


def accumulate_timing_return(total_return_df, realized_period_timing_series):
    trading_day_list = sorted(total_return_df.columns.unique().tolist())
    first_trading_day = trading_day_list[0]
    realized_accumulated_timing_series = pd.Series(index=realized_period_timing_series.index)
    realized_accumulated_timing_series[first_trading_day] = realized_period_timing_series[first_trading_day]

    total_return_dict = total_return_df.to_dict()
    R_T_P = total_return_dict[first_trading_day]['portfolio']
    R_T_B = total_return_dict[first_trading_day]['benchmark']
    C_T = (R_T_P - R_T_B) / (np.log(1 + R_T_P) - np.log(1 + R_T_B))
    if math.isnan(C_T):
        C_T = 0.
    X_t = realized_period_timing_series[first_trading_day]
    for i in range(1, len(trading_day_list)):
        X_pre_t = X_t * i
        X_pre_t_divide_C_T = X_pre_t / C_T if C_T != 0 else 0.

        trading_day = trading_day_list[i]
        timing_t = realized_period_timing_series[trading_day]
        r_p_t = total_return_dict[trading_day]['portfolio']
        r_b_t = total_return_dict[trading_day]['benchmark']
        r_A_t = r_p_t - r_b_t
        alpha_t = timing_t / r_A_t
        if np.isnan(alpha_t):
            alpha_t = 0.
        R_T_P = (1 + R_T_P) * (1 + r_p_t) - 1  # R_T+1_P
        R_T_B = (1 + R_T_B) * (1 + r_b_t) - 1  # R_T+1_B
        C_T = (R_T_P - R_T_B) / (np.log(1 + R_T_P) - np.log(1 + R_T_B))  # C_T+1
        X_t = ((X_pre_t_divide_C_T * C_T) + (C_T * np.log((1 + r_p_t) / (1 + r_b_t)) * alpha_t)) / (i + 1)
        realized_accumulated_timing_series[trading_day] = X_t * (i + 1)

    return realized_accumulated_timing_series


def accumulate_management_return(total_return_df, realized_period_management_df_dict):
    trading_day_list = sorted(realized_period_management_df_dict.keys())
    first_trading_day = trading_day_list[0]
    first_realized_period_management_df = realized_period_management_df_dict[first_trading_day]
    realized_accumulated_management_df_dict = dict()
    realized_accumulated_management_df_dict[first_trading_day] = first_realized_period_management_df

    total_return_dict = total_return_df.to_dict()
    R_T_P = total_return_dict[first_trading_day]['portfolio']
    R_T_B = total_return_dict[first_trading_day]['benchmark']
    C_T = (R_T_P - R_T_B) / (np.log(1 + R_T_P) - np.log(1 + R_T_B))
    if math.isnan(C_T):
        C_T = 0.
    X_t = first_realized_period_management_df
    for i in range(1, len(trading_day_list)):
        X_pre_t = X_t * i
        X_pre_t_divide_C_T = (X_pre_t / C_T).fillna(0.)

        trading_day = trading_day_list[i]
        r_p_t = total_return_dict[trading_day]['portfolio']
        r_b_t = total_return_dict[trading_day]['benchmark']
        r_A_t = r_p_t - r_b_t
        alpha_t = realized_period_management_df_dict[trading_day].divide(r_A_t).fillna(0.)
        R_T_P = (1 + R_T_P) * (1 + r_p_t) - 1  # R_T+1_P
        R_T_B = (1 + R_T_B) * (1 + r_b_t) - 1  # R_T+1_B
        C_T = (R_T_P - R_T_B) / (np.log(1 + R_T_P) - np.log(1 + R_T_B))  # C_T+1
        # if np.isnan(alpha_t):
        #     alpha_t = 0.
        X_t = (X_pre_t_divide_C_T.multiply(C_T)).add(
            C_T * np.log((1 + r_p_t) / (1 + r_b_t)) * alpha_t) / (i + 1)
        realized_accumulated_management_df_dict[trading_day] = X_t * (i + 1)

    return realized_accumulated_management_df_dict



"""
Alpha相关探索研究
"""
from sqlalchemy import create_engine
from hbshare.quant.Kevin.rm_associated.config import engine_params, style_names
from datetime import datetime
import hbshare as hbs
import pandas as pd
from hbshare.quant.Kevin.quant_room.QuarterAssetAllocationReport import plotly_bar
from hbshare.fe.common.util.data_loader import get_trading_day_list
from tqdm import tqdm


class IndexStyleComparer:
    def __init__(self, start_date, end_date):
        self.start_date = start_date
        self.end_date = end_date
        self._load_data()

    def _load_data(self):
        sql_script = "SELECT * FROM stock_index_style_exposure where TRADE_DATE >= {} and TRADE_DATE <= {}".format(
            self.start_date, self.end_date)
        engine = create_engine(engine_params)
        data = pd.read_sql(sql_script, engine)
        data['trade_date'] = data['trade_date'].apply(lambda x: datetime.strftime(x, '%Y%m%d'))
        # exposure_500
        data1 = data[data['benchmark_id'] == '000905']
        data1 = pd.pivot_table(
            data1, index='trade_date', columns='factor_name', values='exposure').sort_index()[style_names]
        # exposure_1000
        data2 = data[data['benchmark_id'] == '000852']
        data2 = pd.pivot_table(
            data2, index='trade_date', columns='factor_name', values='exposure').sort_index()[style_names]
        # factor_return
        sql_script = "SELECT * FROM factor_return where " \
                     "TRADE_DATE >= {} and TRADE_DATE <= {}".format(self.start_date, self.end_date)
        engine = create_engine(engine_params)
        factor_return = pd.read_sql(sql_script, engine)
        factor_return['trade_date'] = factor_return['trade_date'].apply(lambda x: datetime.strftime(x, '%Y%m%d'))
        factor_return = pd.pivot_table(
            factor_return, index='trade_date', columns='factor_name', values='factor_ret').sort_index()[style_names]
        self.factor_return = (1 + factor_return).cumprod().reindex(data1.index).pct_change().shift(-1).dropna()

        self.exposure_500 = data1.reindex(self.factor_return.index)
        self.exposure_1000 = data2.reindex(self.factor_return.index)

    def run(self):
        factor_return = self.factor_return
        exposure_500, exposure_1000 = self.exposure_500, self.exposure_1000

        style_attr = pd.merge(factor_return.multiply(exposure_500).sum().to_frame('中证500'),
                              factor_return.multiply(exposure_1000).sum().to_frame('中证1000'),
                              left_index=True, right_index=True)
        style_attr['差值'] = style_attr['中证500'] - style_attr['中证1000']

        plotly_bar(style_attr, title_text="500指增 vs 1000指增风格收益", save_path="D:\\123.html")


def load_benchmark_components(date):
    sql_script = "SELECT * FROM hsjy_gg.SecuMain where SecuCategory = 4 and " \
                 "SecuCode in ('000300', '000905', '000852')"
    res = hbs.db_data_query('readonly', sql_script)
    index_info = pd.DataFrame(res['data'])
    inner_code_series = index_info.set_index('SECUCODE')['INNERCODE']

    weight = []
    for benchmark_id in ['000300', '000905', '000852']:
        inner_code = inner_code_series.loc[benchmark_id]
        sql_script = "SELECT (select a.SecuCode from hsjy_gg.SecuMain a where a.InnerCode = b.InnerCode and " \
                     "rownum = 1) SecuCode, b.EndDate, b.Weight FROM hsjy_gg.LC_IndexComponentsWeight b WHERE " \
                     "b.IndexCode = '{}' and b.EndDate = to_date('{}', 'yyyymmdd')".format(inner_code, date)
        data = pd.DataFrame(hbs.db_data_query('readonly', sql_script)['data'])
        weight_df = data.rename(
            columns={"SECUCODE": "ticker", "ENDDATE": "effDate", "WEIGHT": "weight"})
        weight_df['benchmark_id'] = benchmark_id
        weight.append(weight_df[['ticker', 'benchmark_id']])

    return pd.concat(weight)


def load_specific_return(date):
    sql_script = "SELECT * FROM st_ashare.r_st_barra_specific_return where TRADE_DATE = '{}'".format(date)
    res = hbs.db_data_query('alluser', sql_script, page_size=5000)
    s_ret = pd.DataFrame(res['data'])
    s_ret['ticker'] = s_ret['ticker'].apply(lambda x: x.zfill(6))
    s_ret = s_ret.set_index('ticker')['s_ret']

    return s_ret


def run_spec(sdate, edate):
    trading_day_list = get_trading_day_list(sdate, edate)
    month_list = get_trading_day_list(sdate, edate, frequency='month')

    all_data = []
    for i in tqdm(range(len(month_list) - 1)):
        month = month_list[i]
        next_month = month_list[i + 1]
        period_list = [x for x in trading_day_list if month < x <= next_month]
        benchmark_weight = load_benchmark_components(month)

        data = []
        for date in period_list:
            spec_ret = load_specific_return(date)
            df = pd.merge(benchmark_weight, spec_ret.to_frame('s_ret'), left_on='ticker', right_index=True, how='right')
            df = df.fillna('other').reset_index(drop=True)

            describe_df = pd.concat([df.groupby('benchmark_id')['s_ret'].mean().to_frame(name='mean'),
                                     df.groupby('benchmark_id')['s_ret'].std().to_frame(name='std'),
                                     df.groupby('benchmark_id')['s_ret'].skew().to_frame(name='skew'),
                                     df.groupby('benchmark_id')['s_ret'].apply(
                                         lambda x: x.kurt()).to_frame(name='kurt')], axis=1)
            describe_df = describe_df.unstack().reset_index()
            describe_df.columns = ['type', 'benchmark_id', 'value']
            describe_df['trade_date'] = date
            data.append(describe_df)
        data = pd.concat(data)
        all_data.append(data)

    all_data = pd.concat(all_data)

    mean_df = pd.pivot_table(
        all_data[all_data['type'] == 'mean'], index='trade_date', columns='benchmark_id', values='value').sort_index()
    std_df = pd.pivot_table(
        all_data[all_data['type'] == 'std'], index='trade_date', columns='benchmark_id', values='value').sort_index()


if __name__ == '__main__':
    IndexStyleComparer('20201220', '20220113').run()
    # run_spec('20201220', '20220113')


"""
线上和本地的净值归因结果的校验程序
"""
import pandas as pd
import hbshare as hbs
from hbshare.fe.nav_attr.nav_attribution import StyleAttribution


class NavAttributionCheck:
    def __init__(self, fund_id, fund_type, start_date, end_date, factor_type, benchmark_id, nav_frequency):
        self.fund_id = fund_id
        self.fund_type = fund_type
        self.start_date = start_date
        self.end_date = end_date
        self.factor_type = factor_type
        self.benchmark_id = benchmark_id
        self.nav_frequency = nav_frequency

    def get_online_res_from_mysql(self, local_res):
        if self.fund_type == 'mutual':
            db_name = 'st_fund'
            user_name = 'funduser'
        else:
            db_name = 'st_hedge'
            user_name = 'highuser'
        output_online = {}
        # exposure_series
        sql_script = "SELECT * FROM {}.r_st_nav_attr_df where jjdm = '{}' and tjrq >= {} and tjrq <= {}".format(
            db_name, self.fund_id, self.start_date, self.end_date)
        res = hbs.db_data_query(user_name, sql_script, page_size=5000)
        data = pd.DataFrame(res['data'])
        data = data[data['attr_type'] == self.factor_type]
        data = data[['data_type', 'style_factor', 'tjrq', 'data_value']]
        output_online['exposure_series'] = pd.pivot_table(
            data[data['data_type'] == 'exposure'],
            index='tjrq', columns='style_factor', values='data_value').sort_index()
        # return_attr_series
        output_online['return_attr_series'] = pd.pivot_table(
            data[data['data_type'] == 'return'],
            index='tjrq', columns='style_factor', values='data_value').sort_index()
        # r_square_series
        output_online['r_square_series'] = data[data['data_type'] == 'r_square'].set_index('tjrq')['data_value']
        # style_allo_bar
        sql_script = "SELECT * FROM {}.r_st_nav_style_allo_bar where jjdm = '{}' and tjrq >= {} and tjrq <= {}".format(
            db_name, self.fund_id, self.start_date, self.end_date)
        res = hbs.db_data_query(user_name, sql_script, page_size=5000)
        data = pd.DataFrame(res['data'])
        data = data[data['attr_type'] == self.factor_type]
        output_online['style_allo_bar'] = data.set_index('tjrq')[['benchmark', 'allocation', 'selection']].sort_index()
        # style_allo_line
        sql_script = "SELECT * FROM {}.r_st_nav_style_allo_line where jjdm = '{}' and tjrq >= {} and tjrq <= {}".format(
            db_name, self.fund_id, self.start_date, self.end_date)
        res = hbs.db_data_query(user_name, sql_script, page_size=5000)
        data = pd.DataFrame(res['data'])
        data = data[data['attr_type'] == self.factor_type]
        tmp = data.set_index('tjrq')[['portfolio', 'benchmark', 'dynamic_style_bm']].sort_index()
        tmp = (1 + tmp).pct_change().dropna()
        output_online['style_allo_line'] = \
            tmp.reindex(local_res['style_allo_line'].index).fillna(0.).add(1.0).cumprod().subtract(1.)

        return output_online

    def compare_results(self):
        local_res = \
            StyleAttribution(fund_id=self.fund_id, fund_type=self.fund_type, start_date=self.start_date,
                             end_date=self.end_date, factor_type=self.factor_type,
                             benchmark_id=self.benchmark_id, nav_frequency=self.nav_frequency).get_all(processed=False)
        online_res = self.get_online_res_from_mysql(local_res)

        return online_res, local_res


if __name__ == '__main__':
    data_ol, data_lc = \
        NavAttributionCheck(fund_id='002943', fund_type='mutual', start_date='20161220', end_date='20220107',
                            factor_type='style_allo', benchmark_id='000985', nav_frequency='day').compare_results()
    print(data_ol['exposure_series'])
    print("\n")
    print(data_lc['exposure_series'].round(6))



#!/usr/bin/python
#coding:utf-8
sql_params = {
    "ip": "192.168.223.152",
    "user": "readonly",
    "pass": "c24mg2e6",
    "port": "3306",
    "database": "riskmodel"
}

engine_params = "mysql+pymysql://{}:{}@{}:{}/{}".format(sql_params['user'], sql_params['pass'], sql_params['ip'],
                                                        sql_params['port'], sql_params['database'])

style_names = ["size", "beta", "momentum", "earnyield", "resvol", "growth", "btop", "leverage", "liquidity", "sizenl"]

industry_names = {
        '银行': 'Bank',
        '房地产': 'RealEstate',
        '医药生物': 'Health',
        '交通运输': 'Transportation',
        '采掘': 'Mining',
        '有色金属': 'NonFerMetal',
        '家用电器': 'HouseApp',
        '休闲服务': 'LeiService',
        '机械设备': 'MachiEquip',
        '建筑装饰': 'BuildDeco',
        '商业贸易': 'CommeTrade',
        '建筑材料': 'CONMAT',
        '汽车': 'Auto',
        '纺织服装': 'Textile',
        '食品饮料': 'FoodBever',
        '电子': 'Electronics',
        '计算机': 'Computer',
        '轻工制造': 'LightIndus',
        '公用事业': 'Utilities',
        '通信': 'Telecom',
        '农林牧渔': 'AgriForest',
        '化工': 'CHEM',
        '传媒': 'Media',
        '钢铁': 'IronSteel',
        '非银金融': 'NonBankFinan',
        '电气设备': 'ELECEQP',
        '国防军工': 'AERODEF',
        '综合': 'Conglomerates'
}

index_name = {"000016": "上证50",
              "000300": "沪深300",
              "000905": "中证500",
              "000852": "中证1000",
              "000985": "上证综指"}



"""
通胀类指标
"""
import pandas as pd
from datetime import datetime
from hbshare.quant.Kevin.asset_allocation.macro_index.util import create_table, delete_duplicate_records, WriteToDB
from WindPy import w

w.start()


class Inflation:
    def __init__(self, start_date, end_date, is_increment=1):
        self.start_date = start_date
        self.end_date = end_date
        self.is_increment = is_increment
        self.table_name = 'mac_inflation'

    def get_inflation_data(self):
        """
        通胀类数据：LME铜、CPI：当月同比、PPI：当月同比
        """
        index_list = ['M0000612', 'M0001227']
        name_dict = {"M0000612": "CPI_yoy", "M0001227": "PPI_yoy"}

        res = w.edb(','.join(index_list), self.start_date, self.end_date)
        if res.ErrorCode != 0:
            data = pd.DataFrame()
            print("fetch inflation data error: start_date = {}, end_date = {}".format(self.start_date, self.end_date))
        else:
            if len(res.Data) == 1:
                data = pd.DataFrame(res.Data[0], index=res.Codes, columns=res.Times).T
            else:
                data = pd.DataFrame(res.Data, index=res.Codes, columns=res.Times).T
            data.index.name = 'trade_date'
            data.reset_index(inplace=True)
            data['trade_date'] = data['trade_date'].apply(lambda x: datetime.strftime(x, '%Y%m%d'))
            data.rename(columns=name_dict, inplace=True)
        inflation = data.copy()
        # CA.LME
        res = w.wsd("CA.LME", "close", self.start_date, self.end_date, "")
        if res.ErrorCode != 0:
            data = pd.DataFrame()
            print("fetch inflation data error: start_date = {}, end_date = {}".format(self.start_date, self.end_date))
        else:
            data = pd.DataFrame(res.Data, index=res.Codes, columns=res.Times).T
            data.index.name = 'trade_date'
            data.reset_index(inplace=True)
            data['trade_date'] = data['trade_date'].apply(lambda x: datetime.strftime(x, '%Y%m%d'))
            data.rename(columns=name_dict, inplace=True)

        data = pd.merge(inflation, data, on='trade_date', how='outer').sort_values(by='trade_date').rename(
            columns={"CA.LME": "LME"})
        data['LME'] = data['LME'].fillna(method='ffill')

        return data

    def get_construct_result(self):
        if self.is_increment == 1:
            data = self.get_inflation_data()
            sql_script = "delete from {} where trade_date in ({})".format(
                self.table_name, ','.join(data['trade_date'].tolist()))
            # delete first
            delete_duplicate_records(sql_script)
            # add new records
            WriteToDB().write_to_db(data, self.table_name)
        else:
            sql_script = """
                create table mac_inflation(
                id int auto_increment primary key,
                trade_date date not null unique,
                CPI_yoy decimal(4, 2),
                PPI_yoy decimal(4, 2),
                LME decimal(7, 2)) 
            """
            create_table(self.table_name, sql_script)
            data = self.get_inflation_data()
            WriteToDB().write_to_db(data, self.table_name)


if __name__ == '__main__':
    Inflation('2005-01-01', '2021-04-23', is_increment=0).get_construct_result()



import pandas as pd
import numpy as np
import datetime
import requests
import json
import hbshare as hbs
from hbshare.fe.common.util.config import style_name, industry_name
from sqlalchemy import create_engine

sql_params = {
    "ip": "192.168.223.152",
    "user": "readonly",
    "pass": "c24mg2e6",
    "port": "3306",
    "database": "daily_data"
}
engine_params = "mysql+pymysql://{}:{}@{}:{}/{}".format(sql_params['user'], sql_params['pass'],
                                                        sql_params['ip'],
                                                        sql_params['port'], sql_params['database'])


def get_trading_day_list(start_date, end_date, frequency='day'):
    sql_script = "SELECT JYRQ, SFJJ, SFZM, SFYM FROM funddb.JYRL WHERE JYRQ >= {} and JYRQ <= {}".format(
        start_date, end_date)
    res = hbs.db_data_query('readonly', sql_script, page_size=5000)
    df = pd.DataFrame(res['data']).rename(
        columns={"JYRQ": 'calendarDate', "SFJJ": 'isOpen',
                 "SFZM": "isWeekEnd", "SFYM": "isMonthEnd"}).sort_values(by='calendarDate')
    df['isOpen'] = df['isOpen'].astype(int).replace({0: 1, 1: 0})
    df['isWeekEnd'] = df['isWeekEnd'].fillna(0).astype(int)
    df['isMonthEnd'] = df['isMonthEnd'].fillna(0).astype(int)

    calendar_df = df[['calendarDate', 'isOpen', 'isWeekEnd', 'isMonthEnd']]

    if frequency == "day":
        trading_day_list = calendar_df[calendar_df['isOpen'] == 1]['calendarDate'].tolist()
    elif frequency == "week":
        trading_day_list = calendar_df[calendar_df['isWeekEnd'] == 1]['calendarDate'].tolist()
    elif frequency == 'month':
        trading_day_list = calendar_df[calendar_df['isMonthEnd'] == 1]['calendarDate'].tolist()
    else:
        trading_day_list = calendar_df['calendarDate'].tolist()

    return trading_day_list


class NavAttributionLoader:
    def __init__(self, fund_id, fund_type, start_date, end_date, factor_type, benchmark_id):
        self.fund_id = fund_id
        self.fund_type = fund_type
        self.start_date = start_date
        self.end_date = end_date
        self.factor_type = factor_type
        self.benchmark_id = benchmark_id
        self._init_api_params()

    def _init_api_params(self):
        self.url = 'http://fdc-query.intelnal.howbuy.com/query/data/commonapi?dataTrack=xxxxx'
        self.headers = {'Content-Type': 'application/json'}
        self.post_body = {"database": "readonly"}

    def fetch_data_batch(self, sql_script):
        post_body = self.post_body.copy()
        post_body['sql'] = sql_script
        post_body["ifByPage"] = False
        res = requests.post(url=self.url, data=json.dumps(post_body), headers=self.headers).json()
        n = res['pages']
        all_data = []
        for i in range(1, n + 1):
            post_body["ifByPage"] = True
            post_body['pageNum'] = i
            res = requests.post(url=self.url, data=json.dumps(post_body), headers=self.headers).json()
            all_data.append(pd.DataFrame(res['data']))
        all_data = pd.concat(all_data)

        return all_data

    @staticmethod
    def fetch_data_batch_hbs(user_name, sql_script):
        total_res = hbs.db_data_query(user_name, sql_script, is_pagination=False)
        n = total_res['pages']
        all_data = []
        for i in range(1, n + 1):
            res = hbs.db_data_query(
                user_name, sql_script, page_num=i, is_pagination=True, page_size=total_res['pageSize'])
            all_data.append(pd.DataFrame(res['data']))
        all_data = pd.concat(all_data)

        return all_data

    def _load_calendar(self):
        sql_script = "SELECT JYRQ, SFJJ, SFZM, SFYM FROM funddb.JYRL WHERE JYRQ >= {} and JYRQ <= {}".format(
            self.start_date, self.end_date)
        post_body = self.post_body
        post_body['sql'] = sql_script
        res = requests.post(url=self.url, data=json.dumps(post_body), headers=self.headers).json()
        df = pd.DataFrame(res['data']).rename(
            columns={"JYRQ": 'calendarDate', "SFJJ": 'isOpen',
                     "SFZM": "isWeekEnd", "SFYM": "isMonthEnd"}).sort_values(by='calendarDate')
        df['isOpen'] = df['isOpen'].astype(int).replace({0: 1, 1: 0})
        df['isWeekEnd'] = df['isWeekEnd'].fillna(0).astype(int)
        df['isMonthEnd'] = df['isMonthEnd'].fillna(0).astype(int)
        df['month'] = df['calendarDate'].apply(lambda x: datetime.datetime.strptime(x, '%Y%m%d').month)
        df['isQuarterEnd'] = np.where((df['isMonthEnd'] == 1) & (df['month'].isin([3, 6, 9, 12])), 1, 0)
        df['isQuarterEnd'] = df['isQuarterEnd'].astype(int)

        return df[['calendarDate', 'isOpen', 'isWeekEnd', 'isMonthEnd', 'isQuarterEnd']]

    def _load_fund_data(self):
        if self.fund_type == 'mutual':
            sql_script = "SELECT a.jjdm fund_id, b.jzrq tradeDate, b.hbcl accumulate_return from " \
                         "funddb.jjxx1 a, funddb.jjhb b where a.cpfl = '2' and a.jjdm = b.jjdm " \
                         "and a.jjzt not in ('3', 'c') " \
                         "and a.m_opt_type <> '03' and a.jjdm = '{}' and b.jzrq >= {} and b.jzrq <= {} " \
                         "order by b.jzrq".format(self.fund_id, self.start_date, self.end_date)
            post_body = self.post_body
            post_body['sql'] = sql_script
            res = requests.post(url=self.url, data=json.dumps(post_body), headers=self.headers).json()
            data = pd.DataFrame(res['data'])
            data['ADJ_NAV'] = 0.01 * data['ACCUMULATE_RETURN'] + 1
        else:
            sql_script = "SELECT a.jjdm fund_id, b.jzrq TRADEDATE, b.fqdwjz as ADJ_NAV from " \
                         "st_hedge.t_st_jjxx a, st_hedge.t_st_rhb b where a.cpfl = '4' and a.jjdm = b.jjdm " \
                         "and a.jjzt not in ('3') " \
                         "and a.jjdm = '{}' and b.jzrq >= {} and b.jzrq <= {} " \
                         "order by b.jzrq".format(self.fund_id, self.start_date, self.end_date)
            res = hbs.db_data_query("highuser", sql_script, page_size=5000)
            data = pd.DataFrame(res['data'])

        data.rename(columns={"FUND_ID": "fund_id", "TRADEDATE": "tradeDate", "ADJ_NAV": "adj_nav"}, inplace=True)

        return data.set_index('tradeDate')['adj_nav']

    def _load_bond_index(self):
        sql_script = "SELECT JYRQ as TRADEDATE, ZQMC as INDEXNAME, SPJG as TCLOSE from funddb.ZSJY WHERE " \
                     "ZQDM = 'H11001' and JYRQ >= {} and JYRQ <= {}".format(self.start_date, self.end_date)
        post_body = self.post_body
        post_body['sql'] = sql_script
        res = requests.post(url=self.url, data=json.dumps(post_body), headers=self.headers).json()
        data = pd.DataFrame(res['data'])
        data.rename(columns={"TCLOSE": "中证全债"}, inplace=True)

        return data.set_index('TRADEDATE')[['中证全债']]

    def _load_factor_data(self):
        factor_type = self.factor_type
        bond_index = self._load_bond_index()
        if factor_type == "style_allo":
            # index_names = ['大盘价值', '大盘成长', '中盘价值', '中盘成长', '小盘价值', '小盘成长']
            index_codes = ['399373', '399372', '399375', '399374', '399377', '399376']
            sql_script = "SELECT jyrq as TRADEDATE, zqmc as INDEXNAME, spjg as TCLOSE from " \
                         "st_market.t_st_zs_hqql WHERE " \
                         "ZQDM in ({}) and JYRQ >= {} and " \
                         "JYRQ <= {}".format(','.join("'{0}'".format(x) for x in index_codes),
                                             self.start_date, self.end_date)
            factor_data = self.fetch_data_batch_hbs('alluser', sql_script)
            factor_data = factor_data.rename(
                columns={"INDEXNAME": "factor_name", "TRADEDATE": "trade_date"})
            # factor_data = pd.DataFrame(hbs.db_data_query('alluser', sql_script, page_size=5000)['data']).rename(
            #     columns={"INDEXNAME": "factor_name", "TRADEDATE": "trade_date"})
            factor_data['trade_date'] = factor_data['trade_date'].map(str)
            factor_data = pd.pivot_table(
                factor_data, index='trade_date', columns='factor_name', values='TCLOSE').sort_index()
            factor_data = pd.merge(factor_data, bond_index, left_index=True, right_index=True)
        elif factor_type == "style":
            # sql_script = "SELECT * FROM factor_return where TRADE_DATE >= {} and TRADE_DATE <= {}".format(
            #     self.start_date, self.end_date)
            # engine = create_engine(engine_params)
            # factor_data = pd.read_sql(sql_script, engine)
            # factor_data['trade_date'] = \
            #     factor_data['trade_date'].apply(lambda x: datetime.datetime.strftime(x, '%Y%m%d'))
            # factor_data = pd.pivot_table(
            #     factor_data, index='trade_date', columns='factor_name', values='factor_ret').sort_index()[style_name]
            # factor_data = (1 + factor_data).cumprod()
            sql_script = "SELECT * FROM st_ashare.r_st_barra_factor_return where " \
                         "TRADE_DATE >= '{}' and TRADE_DATE <= '{}'".format(self.start_date, self.end_date)
            factor_data = self.fetch_data_batch_hbs("alluser", sql_script)
            factor_data = pd.pivot_table(
                factor_data, index='trade_date', columns='factor_name', values='factor_ret').sort_index()[style_name]
            factor_data = (1 + factor_data).cumprod()
        elif factor_type == "sector":
            # sql_script = "SELECT * FROM sector_return where TRADEDATE >= {} and TRADEDATE <= {}".format(
            #     self.start_date, self.end_date)
            # engine = create_engine(engine_params)
            # factor_data = pd.read_sql(sql_script, engine)
            # factor_data['TRADEDATE'] = \
            #     factor_data['TRADEDATE'].apply(lambda x: datetime.datetime.strftime(x, '%Y%m%d'))
            # factor_data.rename(columns={"TRADEDATE": "trade_date", "BIGFINANCE": "大金融",
            #                             "CONSUMING": "消费", "CYCLE": "周期", "MANUFACTURE": "制造"}, inplace=True)
            # factor_data = factor_data.set_index('trade_date').sort_index()
            # del factor_data['id']
            # factor_data = (1 + factor_data).cumprod()
            # factor_data = pd.merge(factor_data, bond_index, left_index=True, right_index=True)
            sql_script = "SELECT * FROM st_market.r_st_sector_factor where " \
                         "trade_date >= {} and trade_date <= {}".format(self.start_date, self.end_date)
            factor_data = self.fetch_data_batch_hbs('alluser', sql_script)
            factor_data = factor_data[['trade_date', 'bigfinance', 'consuming', 'tmt', 'cycle', 'manufacture']]
            factor_data.rename(columns={"bigfinance": "大金融", "consuming": "消费", "tmt": "TMT",
                                        "cycle": "周期", "manufacture": "制造"}, inplace=True)
            factor_data = factor_data.set_index('trade_date').sort_index()
            factor_data = (1 + factor_data).cumprod()
            factor_data = pd.merge(factor_data, bond_index, left_index=True, right_index=True)
        else:
            factor_data = pd.DataFrame()

        return factor_data

    def _load_benchmark_data(self):
        sql_script = "SELECT JYRQ as TRADEDATE, ZQMC as INDEXNAME, SPJG as TCLOSE from funddb.ZSJY WHERE ZQDM = '{}' " \
                     "and JYRQ >= {} and JYRQ <= {}".format(self.benchmark_id, self.start_date, self.end_date)
        post_body = self.post_body
        post_body['sql'] = sql_script
        res = requests.post(url=self.url, data=json.dumps(post_body), headers=self.headers).json()
        data = pd.DataFrame(res['data'])
        data.rename(columns={"TCLOSE": "benchmark"}, inplace=True)

        return data.set_index('TRADEDATE')[['benchmark']]

    def load(self):
        calendar_df = self._load_calendar()
        fund_adj_nav = self._load_fund_data()
        factor_data = self._load_factor_data()
        benchmark_series = self._load_benchmark_data()

        data = {"calendar_df": calendar_df,
                "fund_nav_series": fund_adj_nav,
                "factor_data": factor_data,
                "benchmark_series": benchmark_series}

        return data


class CtaNavAttributionLoader:
    def __init__(self, fund_id, start_date, end_date, style_factors):
        self.fund_id = fund_id
        self.start_date = start_date
        self.end_date = end_date
        self.style_factors = style_factors
        self._pre_date_process()

    def _pre_date_process(self):
        start_dt = datetime.datetime.strptime(self.start_date, '%Y%m%d')
        self.pre_date = (start_dt - datetime.timedelta(days=300)).strftime('%Y%m%d')

    def _load_calendar(self):
        sql_script = "SELECT JYRQ, SFJJ, SFZM, SFYM FROM funddb.JYRL WHERE JYRQ >= {} and JYRQ <= {}".format(
            self.pre_date, self.end_date)
        res = hbs.db_data_query('readonly', sql_script, page_size=5000)
        df = pd.DataFrame(res['data']).rename(
            columns={"JYRQ": 'calendarDate', "SFJJ": 'isOpen',
                     "SFZM": "isWeekEnd", "SFYM": "isMonthEnd"}).sort_values(by='calendarDate')
        df['isOpen'] = df['isOpen'].astype(int).replace({0: 1, 1: 0})
        df['isWeekEnd'] = df['isWeekEnd'].fillna(0).astype(int)
        df['isMonthEnd'] = df['isMonthEnd'].fillna(0).astype(int)

        self.calendar_df = df[['calendarDate', 'isOpen', 'isWeekEnd', 'isMonthEnd']]

        trading_day_list = df[df['isMonthEnd'] == 1]['calendarDate'].tolist()

        return trading_day_list

    @staticmethod
    def fetch_data_batch(user_name, sql_script):
        total_res = hbs.db_data_query(user_name, sql_script, is_pagination=False)
        n = total_res['pages']
        all_data = []
        for i in range(1, n + 1):
            res = hbs.db_data_query(
                user_name, sql_script, page_num=i, is_pagination=True, page_size=total_res['pageSize'])
            all_data.append(pd.DataFrame(res['data']))
        all_data = pd.concat(all_data)

        return all_data

    def _load_fund_data(self):
        sql_script = "SELECT a.jjdm fund_id, b.jzrq TRADEDATE, b.fqdwjz as ADJ_NAV from " \
                     "st_hedge.t_st_jjxx a, st_hedge.t_st_rhb b where a.cpfl = '4' and a.jjdm = b.jjdm " \
                     "and a.jjzt not in ('3') " \
                     "and a.jjdm = '{}' and b.jzrq >= {} and b.jzrq <= {} " \
                     "order by b.jzrq".format(self.fund_id, self.start_date, self.end_date)
        res = hbs.db_data_query("highuser", sql_script, page_size=5000)
        data = pd.DataFrame(res['data']).rename(
            columns={"FUND_ID": "fund_id", "TRADEDATE": "tradeDate", "ADJ_NAV": "adj_nav"})

        return data.set_index('tradeDate')['adj_nav']

    def _load_factor_data(self):
        sql_script = "SELECT * FROM cta_index where TDATE >= {} and TDATE <= {}".format(
            self.pre_date, self.end_date)
        engine = create_engine(engine_params)
        factor_data = pd.read_sql(sql_script, engine).rename(columns={"TDATE": "TRADEDATE"})
        factor_data['TRADEDATE'] = \
            factor_data['TRADEDATE'].apply(lambda x: datetime.datetime.strftime(x, '%Y%m%d'))
        factor_data = pd.pivot_table(
            factor_data, index='TRADEDATE', columns='FACTOR', values='CLOSE').sort_index()[self.style_factors]

        return factor_data

    def load(self):
        trading_day_list = self._load_calendar()
        fund_adj_nav = self._load_fund_data()
        factor_data = self._load_factor_data()

        data = {
            "trading_day_list": trading_day_list,
            "fund_nav_series": fund_adj_nav,
            "factor_data": factor_data}

        return data


class SectorIndexCalculatorLoader:
    def __init__(self, start_date, end_date):
        self.start_date = start_date
        self.end_date = end_date

    def load(self):
        start_dt = datetime.datetime.strptime(self.start_date, '%Y%m%d')
        pre_date = (start_dt - datetime.timedelta(days=30)).strftime('%Y%m%d')

        sql_script = "SELECT * FROM st_market.t_st_zs_hyzsdmdyb where fljb = {} and hyhfbz = 3 and sfyx = 1".format('1')
        res = hbs.db_data_query('alluser', sql_script, page_size=5000)
        data = pd.DataFrame(res['data']).rename(columns={"zsdm": "SYMBOL", "flmc": "INDEXSNAME"})
        map_dict = data.set_index('SYMBOL')['INDEXSNAME'].to_dict()
        industry_index = []
        for key, value in map_dict.items():
            sql_script = "SELECT JYRQ as TRADEDATE, ZQMC as INDEXNAME, SPJG as TCLOSE, " \
                         "LTSZ as NEG_MKV FROM funddb.ZSJY WHERE " \
                         "ZQDM = '{}' and JYRQ >= {} and " \
                         "JYRQ <= {}".format(key, pre_date, self.end_date)
            res = hbs.db_data_query('readonly', sql_script, page_size=5000)
            df = pd.DataFrame(res['data'])
            df['INDEXNAME'] = value
            industry_index.append(df)
        industry_index = pd.concat(industry_index)

        return industry_index


class EquityBrinsonAttributionLoader:
    def __init__(self, fund_id, benchmark_id, start_date, end_date):
        self.fund_id = fund_id
        self.benchmark_id = benchmark_id
        self.start_date = start_date
        self.end_date = end_date

    @staticmethod
    def fetch_data_batch(user_name, sql_script):
        total_res = hbs.db_data_query(user_name, sql_script, is_pagination=False)
        n = total_res['pages']
        all_data = []
        for i in range(1, n + 1):
            res = hbs.db_data_query(
                user_name, sql_script, page_num=i, is_pagination=True, page_size=total_res['pageSize'])
            all_data.append(pd.DataFrame(res['data']))
        all_data = pd.concat(all_data)

        return all_data

    def _load_calendar(self):
        sql_script = "SELECT JYRQ, SFJJ, SFZM, SFYM FROM funddb.JYRL WHERE JYRQ >= {} and JYRQ <= {}".format(
            self.start_date, self.end_date)
        res = hbs.db_data_query('readonly', sql_script, page_size=5000)
        df = pd.DataFrame(res['data']).rename(
            columns={"JYRQ": 'calendarDate', "SFJJ": 'isOpen',
                     "SFZM": "isWeekEnd", "SFYM": "isMonthEnd"}).sort_values(by='calendarDate')
        df['isOpen'] = df['isOpen'].astype(int).replace({0: 1, 1: 0})
        df['isWeekEnd'] = df['isWeekEnd'].fillna(0).astype(int)
        df['isMonthEnd'] = df['isMonthEnd'].fillna(0).astype(int)

        self.calendar_df = df[['calendarDate', 'isOpen', 'isWeekEnd', 'isMonthEnd']]

        df = df[df['isMonthEnd'] == 1].copy()
        trading_day_list = [x for x in sorted(df['calendarDate'].unique()) if x[4:6] in ['06', '12']]

        return trading_day_list

    def _load_portfolio_weight(self):
        sql_script = "SELECT JJDM, JSRQ, ZQDM, ZJBL FROM funddb.GPZH WHERE JJDM = '{}' and GGRQ >= {} " \
                     "and GGRQ <= {}".format(self.fund_id, self.start_date, self.end_date)
        data = self.fetch_data_batch('readonly', sql_script)
        # res = hbs.db_data_query('readonly', sql_script, page_size=5000)
        if data.empty:
            portfolio_weight_series_dict = None
        else:
            data = data.rename(columns={"JSRQ": "endDate", "ZQDM": "ticker", "ZJBL": "weight"})
            # date_list = [x for x in sorted(data['endDate'].unique()) if x[4:6] not in ['03', '09']]
            date_list = [x for x in sorted(data['endDate'].unique()) if x[4:] in ['0630', '1231']]
            trading_day_list = [
                self.calendar_df[(self.calendar_df['calendarDate'] <= x) & (self.calendar_df['isOpen'] == 1)]
                ['calendarDate'].unique()[-1] for x in date_list]
            map_dict = dict(zip(date_list, trading_day_list))
            data['endDate'] = data['endDate'].replace(map_dict)

            portfolio_weight_series_dict = {}
            for date in trading_day_list:
                portfolio_weight_series_dict[date] = \
                    (data[data['endDate'] == date].set_index('ticker')['weight'] / 100.).dropna()

        return portfolio_weight_series_dict

    def _load_benchmark_weight(self, portfolio_weight_series_dict, equity_ratio_series):
        sql_script = "SELECT * FROM hsjy_gg.SecuMain where SecuCategory = 4 and SecuCode = '{}'".format(
            self.benchmark_id)
        res = hbs.db_data_query('readonly', sql_script)
        index_info = pd.DataFrame(res['data'])
        inner_code = index_info.set_index('SECUCODE').loc[self.benchmark_id, 'INNERCODE']
        trading_day_list = sorted(portfolio_weight_series_dict.keys())
        benchmark_weight_series_dict = dict()
        for date in trading_day_list:
            sql_script = "SELECT (select a.SecuCode from hsjy_gg.SecuMain a where a.InnerCode = b.InnerCode and " \
                         "rownum = 1) SecuCode, b.EndDate, b.Weight FROM hsjy_gg.LC_IndexComponentsWeight b WHERE " \
                         "b.IndexCode = '{}' and b.EndDate = to_date('{}', 'yyyymmdd')".format(inner_code, date)
            data = pd.DataFrame(hbs.db_data_query('readonly', sql_script)['data'])
            weight_df = data.rename(
                columns={"SECUCODE": "consTickerSymbol", "ENDDATE": "effDate", "WEIGHT": "weight"})
            benchmark_weight_series_dict[date] = weight_df.set_index(
                'consTickerSymbol')['weight'] / 100. * equity_ratio_series.loc[date]

        return benchmark_weight_series_dict

    @staticmethod
    def _load_security_period_return(portfolio_weight_series_dict, benchmark_weight_series_dict, trading_day_list):
        portfolio_ticker_list = list(pd.DataFrame.from_dict(portfolio_weight_series_dict).index)
        benchmark_ticker_list = list(pd.DataFrame.from_dict(benchmark_weight_series_dict).index)
        ticker_list = sorted(list(set(portfolio_ticker_list).union(set(benchmark_ticker_list))))

        all_data = []
        for trading_day in trading_day_list:
            sql_script = "SELECT (select a.SecuCode from hsjy_gg.SecuMain a where a.InnerCode = b.InnerCode " \
                         "and rownum=1) SecuCode, b.TradingDay, b.BackwardPrice FROM hsjy_gg.QT_PerformanceData b " \
                         "where b.TradingDay = to_date('{}', 'yyyymmdd')".format(trading_day)
            res = hbs.db_data_query('readonly', sql_script, page_size=5000)
            data_main = pd.DataFrame(res['data'])

            sql_script = "SELECT (select a.SecuCode from hsjy_gg.SecuMain a where a.InnerCode = b.InnerCode " \
                         "and rownum=1) SecuCode, b.TradingDay, b.BackwardPrice FROM " \
                         "hsjy_gg.LC_STIBPerformanceData b " \
                         "where b.TradingDay = to_date('{}', 'yyyymmdd')".format(trading_day)
            res = hbs.db_data_query('readonly', sql_script, page_size=5000)
            data_stib = pd.DataFrame(res['data'])

            all_data.append(pd.concat([data_main, data_stib]))

        data = pd.concat(all_data)
        data.rename(
            columns={"SECUCODE": "ticker", "BACKWARDPRICE": "adj_price", "TRADINGDAY": "trade_date"}, inplace=True)
        data = data[data['ticker'].isin(ticker_list)]
        data['trade_date'] = data['trade_date'].apply(
            lambda x: datetime.datetime.strptime(x, '%Y-%m-%d %H:%M:%S').strftime('%Y%m%d'))
        pivot_df = pd.pivot_table(data, index='trade_date', columns='ticker', values='adj_price').sort_index()
        pct_chg = pivot_df.pct_change().dropna(how='all')
        # process
        security_return_series_dict = dict()
        for date in pct_chg.index.tolist():
            security_return_series_dict[date] = pct_chg.loc[date].dropna()

        return security_return_series_dict

    def _load_security_return(self, portfolio_weight_series_dict, benchmark_weight_series_dict):
        df = self.calendar_df[self.calendar_df['isMonthEnd'] == 1].copy()
        trading_day_list = [x for x in sorted(df['calendarDate'].unique()) if x[4:6] in ['06', '12']]
        start_date, end_date = trading_day_list[0], trading_day_list[-1]

        portfolio_ticker_list = list(pd.DataFrame.from_dict(portfolio_weight_series_dict).index)
        benchmark_ticker_list = list(pd.DataFrame.from_dict(benchmark_weight_series_dict).index)
        ticker_list = sorted(list(set(portfolio_ticker_list).union(set(benchmark_ticker_list))))

        n = 100
        sec_return = []
        group_ticker_list = [ticker_list[i: i + n] for i in range(0, len(ticker_list), n)]
        for group_ticker in group_ticker_list:
            sql_script = "SELECT SYMBOL, TDATE, PCHG FROM finchina.CHDQUOTE WHERE" \
                         " SYMBOL in ({}) and TDATE >= {} and TDATE <= {}".format(
                          ','.join("'{0}'".format(x) for x in group_ticker), start_date, end_date)
            data = self.fetch_data_batch('readonly', sql_script)
            sec_return.append(data)
        sec_return = pd.concat(sec_return)
        sec_return['TDATE'] = sec_return['TDATE'].astype(str)
        # process
        security_return_series_dict = dict()
        for i in range(len(trading_day_list) - 1):
            trading_day, next_trading_day = trading_day_list[i], trading_day_list[i + 1]
            period_return = sec_return[(sec_return['TDATE'] > trading_day) & (sec_return['TDATE'] <= next_trading_day)]
            period_return = pd.pivot_table(period_return, index='TDATE', columns='SYMBOL', values='PCHG').sort_index()
            period_return = period_return.fillna(0.) / 100.
            security_return_series_dict[next_trading_day] = (1 + period_return).prod() - 1

        return security_return_series_dict

    @staticmethod
    def _load_security_sector(portfolio_weight_series_dict, benchmark_weight_series_dict):
        trading_day_list = sorted(portfolio_weight_series_dict.keys())

        portfolio_ticker_list = list(pd.DataFrame.from_dict(portfolio_weight_series_dict).index)
        benchmark_ticker_list = list(pd.DataFrame.from_dict(benchmark_weight_series_dict).index)
        ticker_list = sorted(list(set(portfolio_ticker_list).union(set(benchmark_ticker_list))))
        n = 300
        group_ticker_list = [ticker_list[i: i + n] for i in range(0, len(ticker_list), n)]
        cols_list = ['ticker'] + [x.lower() for x in industry_name['sw'].values()]

        security_sector_series_dict = dict()
        for date in trading_day_list:
            all_data = []
            for group_ticker in group_ticker_list:
                sql_script = "SELECT {} FROM st_ashare.r_st_barra_style_factor where TRADE_DATE = '{}' and " \
                             "ticker in ({})".format(','.join(cols_list), date,
                                                     ','.join("'{0}'".format(x) for x in group_ticker))
                res = hbs.db_data_query('alluser', sql_script, page_size=5000)
                data = pd.DataFrame(res['data'])
                all_data.append(data)

            factor_exposure = pd.concat(all_data).set_index('ticker')
            reverse_ind = dict([(value.lower(), key) for (key, value) in industry_name['sw'].items()])
            ind_exposure = factor_exposure[reverse_ind.keys()].rename(columns=reverse_ind)
            ind_exposure = ind_exposure.reset_index().melt(
                id_vars=['ticker'], value_vars=list(reverse_ind.values()), var_name='industryName1', value_name='sign')
            ind_exposure = ind_exposure[ind_exposure['sign'] == '1']
            security_sector_series_dict[date] = ind_exposure.set_index(
                'ticker').reindex(ticker_list)['industryName1'].dropna()

        return security_sector_series_dict

    def _load_nav_based_return(self, trading_day_list):
        nav_based_return_dict = dict()
        for i in range(len(trading_day_list) - 1):
            trading_day, next_trading_day = trading_day_list[i], trading_day_list[i + 1]
            sql_script = "SELECT a.jjdm fund_id, b.jzrq tradeDate, b.hbcl accumulate_return from " \
                         "funddb.jjxx1 a, funddb.jjhb b where a.cpfl = '2' and a.jjdm = b.jjdm " \
                         "and a.jjzt not in ('3', 'c') " \
                         "and a.m_opt_type <> '03' and a.jjdm = '{}' and b.jzrq >= {} and b.jzrq <= {} " \
                         "order by b.jzrq".format(self.fund_id, trading_day, next_trading_day)
            res = hbs.db_data_query('readonly', sql_script, page_size=5000)
            portfolio_nav_df = pd.DataFrame(res['data'])
            if portfolio_nav_df.empty:
                continue
            portfolio_nav_df['ADJ_NAV'] = 0.01 * portfolio_nav_df['ACCUMULATE_RETURN'] + 1
            portfolio_nav_df.rename(columns={"ADJ_NAV": "portfolio"}, inplace=True)

            sql_script = "SELECT JYRQ as TRADEDATE, ZQMC as INDEXNAME, SPJG as TCLOSE from funddb.ZSJY " \
                         "WHERE ZQDM = '{}' and JYRQ >= {} and " \
                         "JYRQ <= {}".format(self.benchmark_id, trading_day, next_trading_day)
            res = hbs.db_data_query('readonly', sql_script, page_size=5000)
            equity_benchmark_nav_df = pd.DataFrame(res['data'])
            equity_benchmark_nav_df.rename(columns={"TCLOSE": "equity_bm"}, inplace=True)

            sql_script = "SELECT JYRQ as TRADEDATE, ZQMC as INDEXNAME, SPJG as TCLOSE from funddb.ZSJY " \
                         "WHERE ZQDM = '{}' and JYRQ >= {} and " \
                         "JYRQ <= {}".format('H11001', trading_day, next_trading_day)
            res = hbs.db_data_query('readonly', sql_script, page_size=5000)
            bond_benchmark_nav_df = pd.DataFrame(res['data'])
            bond_benchmark_nav_df.rename(columns={"TCLOSE": "bond_bm"}, inplace=True)

            nav_df = portfolio_nav_df[['TRADEDATE', 'portfolio']].merge(
                equity_benchmark_nav_df[['TRADEDATE', 'equity_bm']], on='TRADEDATE').merge(
                bond_benchmark_nav_df[['TRADEDATE', 'bond_bm']], on='TRADEDATE')

            nav_based_return_dict[next_trading_day] = \
                (nav_df.set_index('TRADEDATE').pct_change().dropna() + 1).prod() - 1

        return nav_based_return_dict

    def _load_portfolio_equity_ratio(self, N=12):
        sql_script = "SELECT JJDM, JSRQ, GPBL FROM funddb.ZCPZ WHERE JJDM = '{}'".format(self.fund_id)
        res = hbs.db_data_query('readonly', sql_script, page_size=5000)
        data = pd.DataFrame(res['data']).rename(
            columns={"JSRQ": "endDate", "GPBL": "equity_ratio"})[['JJDM', 'endDate', 'equity_ratio']].dropna()
        data['equity_ratio'] /= 100.
        data['avg_ratio'] = data['equity_ratio'].rolling(N, min_periods=1).mean()
        data = data[(data['endDate'] >= self.start_date) & (data['endDate'] <= self.end_date)]

        date_list = [x for x in sorted(data['endDate'].unique()) if x[4:6] not in ['03', '09']]
        trading_day_list = [
            self.calendar_df[(self.calendar_df['calendarDate'] <= x) & (self.calendar_df['isOpen'] == 1)]
            ['calendarDate'].unique()[-1] for x in date_list]
        map_dict = dict(zip(date_list, trading_day_list))
        data['endDate'] = data['endDate'].replace(map_dict)
        data = data[data['endDate'].isin(trading_day_list)]

        return data.set_index('endDate')['avg_ratio']

    def load(self):
        trading_day_list = self._load_calendar()
        if len(trading_day_list) < 2:
            return
        portfolio_weight_series_dict = self._load_portfolio_weight()
        if not portfolio_weight_series_dict:
            return
        # tdl adjust
        weight_start_date = sorted(portfolio_weight_series_dict.keys())[0]
        trading_day_list = [x for x in trading_day_list if x >= weight_start_date]

        equity_ratio_series = self._load_portfolio_equity_ratio()
        benchmark_weight_series_dict = self._load_benchmark_weight(portfolio_weight_series_dict, equity_ratio_series)
        security_sector_series_dict = self._load_security_sector(
            portfolio_weight_series_dict, benchmark_weight_series_dict)

        security_return_series_dict = self._load_security_period_return(
            portfolio_weight_series_dict, benchmark_weight_series_dict, trading_day_list)
        nav_based_return_dict = self._load_nav_based_return(trading_day_list)

        data_param = {
            "trading_day_list": trading_day_list,
            "portfolio_weight_series_dict": portfolio_weight_series_dict,
            "benchmark_weight_series_dict": benchmark_weight_series_dict,
            "security_sector_series_dict": security_sector_series_dict,
            "security_return_series_dict": security_return_series_dict,
            "nav_based_return_dict": nav_based_return_dict}

        return data_param


class HoldingAttributionLoader:
    """
    先前的持仓Loader
    """
    def __init__(self, fund_id, benchmark_id, start_date, end_date, mode='style'):
        self.fund_id = fund_id
        self.benchmark_id = benchmark_id
        self.start_date = start_date
        self.end_date = end_date
        self.mode = mode

    @staticmethod
    def fetch_data_batch(user_name, sql_script):
        total_res = hbs.db_data_query(user_name, sql_script, is_pagination=False)
        n = total_res['pages']
        all_data = []
        for i in range(1, n + 1):
            res = hbs.db_data_query(
                user_name, sql_script, page_num=i, is_pagination=True, page_size=total_res['pageSize'])
            all_data.append(pd.DataFrame(res['data']))
        all_data = pd.concat(all_data)

        return all_data

    def _load_calendar(self):
        sql_script = "SELECT JYRQ, SFJJ, SFZM, SFYM FROM funddb.JYRL WHERE JYRQ >= {} and JYRQ <= {}".format(
            self.start_date, self.end_date)
        res = hbs.db_data_query('readonly', sql_script, page_size=5000)
        df = pd.DataFrame(res['data']).rename(
            columns={"JYRQ": 'calendarDate', "SFJJ": 'isOpen',
                     "SFZM": "isWeekEnd", "SFYM": "isMonthEnd"}).sort_values(by='calendarDate')
        df['isOpen'] = df['isOpen'].astype(int).replace({0: 1, 1: 0})
        df['isWeekEnd'] = df['isWeekEnd'].fillna(0).astype(int)
        df['isMonthEnd'] = df['isMonthEnd'].fillna(0).astype(int)

        self.calendar_df = df[['calendarDate', 'isOpen', 'isWeekEnd', 'isMonthEnd']]

    def _load_portfolio_weight(self):
        sql_script = "SELECT JJDM, JSRQ, ZQDM, ZJBL FROM funddb.GPZH WHERE JJDM = '{}' and GGRQ >= {} " \
                     "and GGRQ <= {}".format(self.fund_id, self.start_date, self.end_date)
        res = hbs.db_data_query('readonly', sql_script, page_size=5000)
        data = pd.DataFrame(res['data']).rename(
            columns={"JSRQ": "endDate", "ZQDM": "ticker", "ZJBL": "weight"})
        date_list = [x for x in sorted(data['endDate'].unique()) if x[4:6] not in ['03', '09']]
        trading_day_list = [
            self.calendar_df[(self.calendar_df['calendarDate'] <= x) & (self.calendar_df['isOpen'] == 1)]
            ['calendarDate'].unique()[-1] for x in date_list]
        map_dict = dict(zip(date_list, trading_day_list))
        data['endDate'] = data['endDate'].replace(map_dict)

        portfolio_weight_series_dict = {}
        for date in trading_day_list:
            portfolio_weight_series_dict[date] = data[data['endDate'] == date].set_index('ticker')['weight'] / 100.

        return trading_day_list, portfolio_weight_series_dict

    @staticmethod
    def _load_risk_model_data(trading_day_list):
        risk_model_dict = dict()
        risk_model_dict['schema'] = {"industry_field": sorted(pd.Series(industry_name['sw']).values.tolist()),
                                     "style_field": style_name + ['country']}
        factor_order = style_name + risk_model_dict['schema']['industry_field'] + ['country']
        data = dict()
        for date in trading_day_list[:-1]:
            sql_script = "SELECT * FROM st_ashare.r_st_barra_style_factor where TRADE_DATE = '{}'".format(date)
            res = hbs.db_data_query('alluser', sql_script, page_size=5000)
            factor_exposure = pd.DataFrame(res['data']).set_index('ticker')
            exclude = ['credt_etl', 'moddt_etl', 'id', 'trade_date', 'm_opt_type']
            cols = sorted([x for x in factor_exposure.columns if x not in exclude])
            map_dict = {x.lower(): x for x in risk_model_dict['schema']['industry_field']}
            factor_exposure = factor_exposure[cols].rename(columns=map_dict)[factor_order]

            sql_script = "SELECT * FROM st_ashare.r_st_barra_factor_cov where TRADE_DATE = '{}'".format(date)
            res = hbs.db_data_query('alluser', sql_script, page_size=5000)
            factor_covariance = pd.DataFrame(res['data']).set_index('factor_name')
            factor_covariance = factor_covariance[cols].rename(columns=map_dict).reindex(factor_order)[factor_order]

            sql_script = "SELECT * FROM st_ashare.r_st_barra_s_risk where TRADE_DATE = '{}'".format(date)
            res = hbs.db_data_query('alluser', sql_script, page_size=5000)
            srisk = pd.DataFrame(res['data']).set_index('ticker')['s_ret']

            data[date] = {"exposure": factor_exposure, "factor_covariance": factor_covariance, "specific_risk": srisk}

        risk_model_dict['data'] = data

        return risk_model_dict

    def _load_factor_return(self, trading_day_list):
        factor_return_series_dict = dict()
        for i in range(len(trading_day_list) - 1):
            trading_day, next_trading_day = trading_day_list[i], trading_day_list[i + 1]
            sql_script = "SELECT * FROM st_ashare.r_st_barra_factor_return where " \
                         "TRADE_DATE > '{}' and TRADE_DATE <= {}".format(trading_day, next_trading_day)
            data = self.fetch_data_batch('alluser', sql_script)
            factor_return = pd.pivot_table(
                data, index='trade_date', columns='factor_name', values='factor_ret').sort_index()
            factor_return_series_dict[next_trading_day] = (1 + factor_return).prod() - 1

        return factor_return_series_dict

    def _load_benchmark_weight(self, trading_day_list):
        benchmark_weight_series_dict = dict()
        for date in trading_day_list:
            sql_script = "SELECT a.EndDate, a.Weight, c.SecuCode " \
                         "FROM hsjy_gg.LC_IndexComponentsWeight a, hsjy_gg.SecuMain b, hsjy_gg.SecuMain c " \
                         "WHERE a.indexCode = b.innerCode and b.SecuCode = '{}' and " \
                         "to_char(a.EndDate, 'yyyymmdd') = '{}' and b.SecuCategory = 4 " \
                         "and a.InnerCode = c.InnerCode".format(self.benchmark_id, date)
            data = self.fetch_data_batch('readonly', sql_script)
            weight_df = data.rename(
                columns={"SECUCODE": "consTickerSymbol", "ENDDATE": "effDate", "WEIGHT": "weight"})
            benchmark_weight_series_dict[date] = weight_df.set_index('consTickerSymbol')['weight'] / 100.

        return benchmark_weight_series_dict

    def _load_security_return(self, portfolio_weight_series_dict, benchmark_weight_series_dict):
        trading_day_list = sorted(portfolio_weight_series_dict.keys())
        start_date, end_date = trading_day_list[0], trading_day_list[-1]

        portfolio_ticker_list = list(pd.DataFrame.from_dict(portfolio_weight_series_dict).index)
        benchmark_ticker_list = list(pd.DataFrame.from_dict(benchmark_weight_series_dict).index)
        ticker_list = sorted(list(set(portfolio_ticker_list).union(set(benchmark_ticker_list))))

        n = 100
        sec_return = []
        group_ticker_list = [ticker_list[i: i + n] for i in range(0, len(ticker_list), n)]
        for group_ticker in group_ticker_list:
            sql_script = "SELECT SYMBOL, TDATE, PCHG FROM finchina.CHDQUOTE WHERE" \
                         " SYMBOL in ({}) and TDATE >= {} and TDATE <= {}".format(
                          ','.join("'{0}'".format(x) for x in group_ticker), start_date, end_date)
            data = self.fetch_data_batch('readonly', sql_script)
            sec_return.append(data)
        sec_return = pd.concat(sec_return)
        sec_return['TDATE'] = sec_return['TDATE'].astype(str)
        # process
        security_return_series_dict = dict()
        for i in range(len(trading_day_list) - 1):
            trading_day, next_trading_day = trading_day_list[i], trading_day_list[i + 1]
            period_return = sec_return[(sec_return['TDATE'] > trading_day) & (sec_return['TDATE'] <= next_trading_day)]
            period_return = pd.pivot_table(period_return, index='TDATE', columns='SYMBOL', values='PCHG').sort_index()
            period_return = period_return.fillna(0.) / 100.
            security_return_series_dict[next_trading_day] = (1 + period_return).prod() - 1

        return security_return_series_dict

    @staticmethod
    def _load_security_sector(portfolio_weight_series_dict, benchmark_weight_series_dict):
        trading_day_list = sorted(portfolio_weight_series_dict.keys())

        portfolio_ticker_list = list(pd.DataFrame.from_dict(portfolio_weight_series_dict).index)
        benchmark_ticker_list = list(pd.DataFrame.from_dict(benchmark_weight_series_dict).index)
        ticker_list = sorted(list(set(portfolio_ticker_list).union(set(benchmark_ticker_list))))

        security_sector_series_dict = dict()
        for date in trading_day_list:
            sql_script = "SELECT * FROM st_ashare.r_st_barra_style_factor where TRADE_DATE = '{}'".format(date)
            res = hbs.db_data_query('alluser', sql_script, page_size=5000)
            factor_exposure = pd.DataFrame(res['data']).set_index('ticker')
            reverse_ind = dict([(value.lower(), key) for (key, value) in industry_name['sw'].items()])
            ind_exposure = factor_exposure[reverse_ind.keys()].rename(columns=reverse_ind)
            ind_exposure = ind_exposure.reset_index().melt(
                id_vars=['ticker'], value_vars=list(reverse_ind.values()), var_name='industryName1', value_name='sign')
            ind_exposure = ind_exposure[ind_exposure['sign'] == '1']
            security_sector_series_dict[date] = ind_exposure.set_index(
                'ticker').reindex(ticker_list)['industryName1'].dropna()

        return security_sector_series_dict

    def _load_nav_active_return(self, trading_day_list):
        nav_active_return_dict = dict()
        for i in range(len(trading_day_list) - 1):
            trading_day, next_trading_day = trading_day_list[i], trading_day_list[i + 1]
            sql_script = "SELECT a.jjdm fund_id, b.jzrq tradeDate, b.hbcl accumulate_return from " \
                         "funddb.jjxx1 a, funddb.jjhb b where a.cpfl = '2' and a.jjdm = b.jjdm " \
                         "and a.jjzt not in ('3', 'c') " \
                         "and a.m_opt_type <> '03' and a.jjdm = '{}' and b.jzrq >= {} and b.jzrq <= {} " \
                         "order by b.jzrq".format(self.fund_id, trading_day, next_trading_day)
            nav_df = self.fetch_data_batch('readonly', sql_script)
            nav_df['ADJ_NAV'] = 0.01 * nav_df['ACCUMULATE_RETURN'] + 1

            sql_script = "SELECT JYRQ as TRADEDATE, ZQMC as INDEXNAME, SPJG as TCLOSE from funddb.ZSJY WHERE ZQDM = '{}' " \
                         "and JYRQ >= {} and JYRQ <= {}".format(self.benchmark_id, trading_day, next_trading_day)
            benchmark_nav_df = self.fetch_data_batch('readonly', sql_script)

            nav_df = pd.merge(
                nav_df[['TRADEDATE', 'ADJ_NAV']], benchmark_nav_df[['TRADEDATE', 'TCLOSE']], on='TRADEDATE').rename(
                columns={"ADJ_NAV": "portfolio", "TCLOSE": "benchmark"})

            nav_active_return_dict[next_trading_day] = \
                (nav_df.set_index('TRADEDATE').pct_change().dropna() + 1).prod() - 1

        return nav_active_return_dict

    def load(self):
        self._load_calendar()
        trading_day_list, portfolio_weight_series_dict = self._load_portfolio_weight()
        benchmark_weight_series_dict = self._load_benchmark_weight(trading_day_list)
        security_return_series_dict = self._load_security_return(
            portfolio_weight_series_dict, benchmark_weight_series_dict)
        if self.mode == "style":
            factor_return_series_dict = self._load_factor_return(trading_day_list)
            risk_model_dict = self._load_risk_model_data(trading_day_list)
            security_sector_series_dict = None
            nav_active_return_dict = None
        else:
            factor_return_series_dict = None
            risk_model_dict = None
            security_sector_series_dict = self._load_security_sector(
                portfolio_weight_series_dict, benchmark_weight_series_dict)
            nav_active_return_dict = self._load_nav_active_return(trading_day_list)

        data_param = {
            "trading_day_list": trading_day_list,
            "portfolio_weight_series_dict": portfolio_weight_series_dict,
            "risk_model_dict": risk_model_dict,
            "benchmark_weight_series_dict": benchmark_weight_series_dict,
            "security_return_series_dict": security_return_series_dict,
            "factor_return_series_dict": factor_return_series_dict,
            "security_sector_series_dict": security_sector_series_dict,
            "nav_active_return_dict": nav_active_return_dict}

        return data_param


class MorningStarBoxStyleLoader:
    def __init__(self, trade_date, mode):
        self.trade_date = trade_date
        self.mode = mode

    @staticmethod
    def fetch_data_batch(user_name, sql_script):
        total_res = hbs.db_data_query(user_name, sql_script, is_pagination=False)
        n = total_res['pages']
        all_data = []
        for i in range(1, n + 1):
            res = hbs.db_data_query(
                user_name, sql_script, page_num=i, is_pagination=True, page_size=total_res['pageSize'])
            all_data.append(pd.DataFrame(res['data']))
        all_data = pd.concat(all_data)

        return all_data

    def _load_associated_date(self):
        pre_date = str(int(self.trade_date[:4]) - 2) + '0101'
        sql_script = "SELECT JYRQ, SFJJ, SFZM, SFYM FROM funddb.JYRL WHERE JYRQ >= {} and JYRQ <= {}".format(
           pre_date, self.trade_date)
        res = hbs.db_data_query('readonly', sql_script, page_size=5000)
        df = pd.DataFrame(res['data']).rename(
            columns={"JYRQ": 'calendarDate', "SFJJ": 'isOpen',
                     "SFZM": "isWeekEnd", "SFYM": "isMonthEnd"}).sort_values(by='calendarDate')
        df['isMonthEnd'] = df['isMonthEnd'].fillna(0).astype(int)
        df = df[df['isMonthEnd'] == 1]

        if self.mode == "main":  # quarter
            df['pre_end'] = df['calendarDate'].shift(1)
            quotation_date = df.set_index('calendarDate').loc[self.trade_date, 'pre_end']
            report_date = \
                quotation_date[:6] + '30' if self.trade_date[4:6] in ["07", "10"] else quotation_date[:6] + "31"
        else:
            df['shift_2'] = df['calendarDate'].shift(2)
            df['shift_4'] = df['calendarDate'].shift(4)
            if self.trade_date[4:6] == "08":
                quotation_date = df.set_index('calendarDate').loc[self.trade_date, 'shift_2']
                report_date = quotation_date[:6] + '30'
            else:
                quotation_date = df.set_index('calendarDate').loc[self.trade_date, 'shift_4']
                report_date = quotation_date[:6] + '31'

        return quotation_date, report_date

    def _load_valuation_data(self, date):
        """
        估值数据
        """
        # 主板
        sql_script = "SELECT TotalMV, NegotiableMV, PE, PB, PCFTTM, DividendRatio, SecuCode FROM " \
                     "(SELECT b.TotalMV, b.NegotiableMV, b.PE, b.PB, b.PCFTTM, b.DividendRatio, a.SecuCode, " \
                     "row_number() over(partition by a.InnerCode order by a.InnerCode) rn FROM " \
                     "hsjy_gg.LC_DIndicesForValuation b join hsjy_gg.SecuMain a on a.InnerCode = b.InnerCode and " \
                     "a.SecuMarket in (83, 90) and a.SecuCategory = 1 WHERE b.TradingDay = to_date('{}', 'yyyymmdd')) " \
                     "WHERE rn = 1".format(date)
        data_main = self.fetch_data_batch('readonly', sql_script)
        data_main.rename(columns={"SECUCODE": "ticker", "TOTALMV": "marketValue",
                                  "NEGOTIABLEMV": "negMarketValue", "PCFTTM": "PCF"}, inplace=True)
        # 科创板
        sql_script = "SELECT TotalMV, NegotiableMV, PETTM, PB, PCFTTM, DividendRatio, SecuCode FROM " \
                     "(SELECT b.TotalMV, b.NegotiableMV, b.PETTM, b.PB, b.PCFTTM, b.DividendRatio, a.SecuCode, " \
                     "row_number() over(partition by a.InnerCode order by a.InnerCode) rn FROM " \
                     "hsjy_gg.LC_STIBDIndiForValue b join hsjy_gg.SecuMain a on a.InnerCode = b.InnerCode and " \
                     "a.SecuMarket in (83, 90) and a.SecuCategory = 1 WHERE b.TradingDay = to_date('{}', 'yyyymmdd')) " \
                     "WHERE rn = 1".format(date)
        res = hbs.db_data_query('readonly', sql_script)
        data_stib = pd.DataFrame(res['data'])
        data_stib.rename(
            columns={"SECUCODE": "ticker", "TOTALMV": "marketValue",
                     "NEGOTIABLEMV": "negMarketValue", "PETTM": "PE", "PCFTTM": "PCF"}, inplace=True)

        data = pd.concat([data_main, data_stib])
        del data['ROW_ID']
        data = data.dropna(subset=['ticker'])

        return data.set_index('ticker')

    def _load_growth_data(self):
        """
        成长数据
        """
        last_fiscal_year = int(self.trade_date[:4])
        last_fiscal_year = last_fiscal_year - 1 if self.trade_date[4:6] == '01' else last_fiscal_year
        fiscal_year_list = [str(x) + '1231' for x in np.arange(last_fiscal_year - 3, last_fiscal_year)]

        sql_script = "SELECT SYMBOL, COMPCODE FROM finchina.TQ_OA_STCODE WHERE SETYPE in (101, 108)"
        s_code = self.fetch_data_batch('readonly', sql_script)
        ticker_list = s_code['COMPCODE'].unique()
        n = 300
        group_ticker_list = [ticker_list[i: i + n] for i in range(0, len(ticker_list), n)]
        # 利润表
        data = []
        for group_ticker in group_ticker_list:
            # sql_script = "SELECT COMPCODE, ENDDATE, PUBLISHDATE, BIZINCO, NETPROFIT FROM " \
            #              "finchina.TQ_FIN_PROINCSTATEMENTNEW WHERE REPORTTYPE in (1, 3) and " \
            #              "ENDDATE in ({}) and PUBLISHDATE <= '{}' and " \
            #              "COMPCODE in ({})".format(','.join(fiscal_year_list), self.trade_date,
            #                                        ','.join("'{0}'".format(x) for x in group_ticker))
            sql_script = "SELECT COMPCODE, ENDDATE, PUBLISHDATE, BIZINCO, NETPROFIT FROM " \
                         "finchina.TQ_FIN_PROINCSTATEMENTNEW WHERE REPORTTYPE in (1, 3) and " \
                         "ENDDATE in ({}) and " \
                         "COMPCODE in ({})".format(','.join(fiscal_year_list),
                                                   ','.join("'{0}'".format(x) for x in group_ticker))
            res = hbs.db_data_query('readonly', sql_script, page_size=5000)
            data.append(pd.DataFrame(res['data']))

        data = pd.concat(data).sort_values(by=['COMPCODE', 'ENDDATE', 'PUBLISHDATE'])
        data.drop_duplicates(subset=['COMPCODE', 'ENDDATE'], keep='last', inplace=True)
        revenue_data = data[['COMPCODE', 'ENDDATE', 'BIZINCO', 'NETPROFIT']]
        # 资产负债表
        data = []
        for group_ticker in group_ticker_list:
            # sql_script = "SELECT COMPCODE, ENDDATE, PUBLISHDATE, TOTASSET FROM " \
            #              "finchina.TQ_FIN_PROBALSHEETNEW WHERE REPORTTYPE in (1, 3) and " \
            #              "ENDDATE in ({}) and PUBLISHDATE <= '{}' and " \
            #              "COMPCODE in ({})".format(','.join(fiscal_year_list), self.trade_date,
            #                                        ','.join("'{0}'".format(x) for x in group_ticker))
            sql_script = "SELECT COMPCODE, ENDDATE, PUBLISHDATE, TOTASSET FROM " \
                         "finchina.TQ_FIN_PROBALSHEETNEW WHERE REPORTTYPE in (1, 3) and " \
                         "ENDDATE in ({}) and " \
                         "COMPCODE in ({})".format(','.join(fiscal_year_list),
                                                   ','.join("'{0}'".format(x) for x in group_ticker))
            res = hbs.db_data_query('readonly', sql_script, page_size=5000)
            data.append(pd.DataFrame(res['data']))
        data = pd.concat(data).sort_values(by=['COMPCODE', 'ENDDATE', 'PUBLISHDATE'])
        data.drop_duplicates(subset=['COMPCODE', 'ENDDATE'], keep='last', inplace=True)
        asset_data = data[['COMPCODE', 'ENDDATE', 'TOTASSET']]

        fin_data = pd.merge(revenue_data, asset_data, on=['COMPCODE', 'ENDDATE']).dropna()
        fin_data = pd.merge(fin_data, s_code[['SYMBOL', 'COMPCODE']], on='COMPCODE').rename(
            columns={"SYMBOL": "ticker", "ENDDATE": "endDate", "BIZINCO": "revenue",
                     "NETPROFIT": "net_profit", "TOTASSET": "t_asset"})

        tmp = fin_data['ticker'].value_counts()
        tmp = tmp[tmp == 3]
        fin_data = fin_data[fin_data['ticker'].isin(tmp.index)]

        return fin_data[['ticker', 'endDate', 'revenue', 'net_profit', 't_asset']]

    def _load_fund_holding_data(self, date):
        sql_script = "select jjdm, clrq, zzrq, (select count(1) from st_fund.t_st_gm_rhb r where r.jjdm = a.jjdm) " \
                     "hb_count from st_fund.t_st_gm_jjxx a " \
                     "where a.Jjfl in ('1','3') and m_opt_type <> '03' and clrq is not null " \
                     "order by clrq asc"
        data = self.fetch_data_batch('funduser', sql_script)
        data['clrq'] = data['clrq'].map(str)
        data['zzrq'] = data['zzrq'].fillna('20991231')

        # 成立满一年
        begin_dt = datetime.datetime.strptime(self.trade_date, '%Y%m%d') - datetime.timedelta(days=365)
        begin_date = datetime.datetime.strftime(begin_dt, '%Y%m%d')
        fund_id_list = data[(data['clrq'] < begin_date) & (data['zzrq'] > self.trade_date)]['jjdm'].tolist()

        n = 50
        fund_holding = []
        group_fund_id_list = [fund_id_list[i: i + n] for i in range(0, len(fund_id_list), n)]
        for group_id in group_fund_id_list:
            sql_script = "SELECT JJDM, ZQDM, JSRQ, GGRQ, ZJBL FROM funddb.GPZH WHERE JJDM in ({}) and JSRQ = {} and " \
                         "GGRQ <= {}".format(','.join("'{0}'".format(x) for x in group_id), date, self.trade_date)
            df = self.fetch_data_batch('readonly', sql_script)
            fund_holding.append(df)

        fund_holding = pd.concat(fund_holding)
        # holding num >= 10
        count_df = fund_holding['JJDM'].value_counts()
        count_df = count_df[count_df >= 10]
        fund_holding = fund_holding[fund_holding['JJDM'].isin(count_df.index)]
        fund_holding.rename(columns={"JJDM": "fund_id", "ZQDM": "ticker", "ZJBL": "weight"}, inplace=True)
        fund_holding['weight'] /= 100.

        return fund_holding[['fund_id', 'ticker', 'weight']]

    def load(self):
        quotation_date, report_date = self._load_associated_date()
        equity_value = self._load_valuation_data(quotation_date)
        equity_growth = self._load_growth_data()
        fund_holding_df = self._load_fund_holding_data(report_date)

        return {
            "quotation_date": quotation_date,
            "report_date": report_date,
            "equity_value": equity_value,
            "equity_growth": equity_growth,
            "fund_holding": fund_holding_df
        }


def get_nav_data_from_mysql(fund_name, fof_name):
    sql_script = "SELECT jjdm as fund_id, jjmc as fund_name, khmc as fof_name, " \
                 "jzrq as trade_date, jjjz, ljjz, xnjz, khzcfe FROM ac_fof.t_ac_fof_xfc where " \
                 "jjmc = '{}' and khmc = '{}' order by jzrq".format(fund_name, fof_name)
    res = hbs.db_data_query('alluser', sql_script)
    data = pd.DataFrame(res['data'])

    return data
